(* Script to create an md5 checksum file at the same location as the checksummed file

ToDo:

- escape quotes in filenames, because that will result in an error
- do the fileCount over all, not only every one selected file (multiple dialogs)
- optional crc using "cksum" instead of md5 (cksum only on Snow Leopard (10.6) and maybe higher)

*)

(* Global Vars *)
global fileCount, logMessage

(* Properties (English) *)
property dialogTooMuchFilesMessage01English : "Are you sure you want to create "
property dialogTooMuchFilesMessage02English : " checksum files?"
property dialogTooMuchFilesYesEnglish : "Yes, sure!"
property dialogTooMuchFilesNoEnglish : "Hell, no!"
property dialogNothingSelectedMessageEnglish : "Please select one or more files in Finder."
property dialogNothingSelectedButtonEnglish : "Allright, I do whatever you say!"
property dialogErrorOccurredMessageEnglish : "One or more errors occured: "
property dialogErrorOccurredButtonEnglish : "Doh!"
(* Properties (Deutsch) *)
property dialogTooMuchFilesMessage01German : "Bist Du sicher, dass Du "
property dialogTooMuchFilesMessage02German : " Prfsummen-Dateien erzeugen willst?"
property dialogTooMuchFilesYesGerman : "Na klar!"
property dialogTooMuchFilesNoGerman : "Bloss nicht!"
property dialogNothingSelectedMessageGerman : "Bitte whle eine oder mehrere Dateien im Finder aus."
property dialogNothingSelectedButtonGerman : "Okay, mach ich!"
property dialogErrorOccurredMessageGerman : "Ein oder mehr Fehler ist aufgetreten: "
property dialogErrorOccurredButtonGerman : "Och n!"

(* UnMain() *)
on run (argv)
	(* Init *)
	set logMessage to ""
	if (my systemLanguage() is "de") then
		set dialogTooMuchFilesMessage01 to dialogTooMuchFilesMessage01German
		set dialogTooMuchFilesMessage02 to dialogTooMuchFilesMessage02German
		set dialogTooMuchFilesYes to dialogTooMuchFilesYesGerman
		set dialogTooMuchFilesNo to dialogTooMuchFilesNoGerman
		set dialogNothingSelectedMessage to dialogNothingSelectedMessageGerman
		set dialogNothingSelectedButton to dialogNothingSelectedButtonGerman
		set dialogErrorOccurredMessage to dialogErrorOccurredMessageGerman
		set dialogErrorOccurredButton to dialogErrorOccurredButtonGerman
	else
		set dialogTooMuchFilesMessage01 to dialogTooMuchFilesMessage01English
		set dialogTooMuchFilesMessage02 to dialogTooMuchFilesMessage02English
		set dialogTooMuchFilesYes to dialogTooMuchFilesYesEnglish
		set dialogTooMuchFilesNo to dialogTooMuchFilesNoEnglish
		set dialogNothingSelectedMessage to dialogNothingSelectedMessageEnglish
		set dialogNothingSelectedButton to dialogNothingSelectedButtonEnglish
		set dialogErrorOccurredMessage to dialogErrorOccurredMessageEnglish
		set dialogErrorOccurredButton to dialogErrorOccurredButtonEnglish
	end if
	
	(* Get selected files/folders *)
	tell application "Finder"
	activate
		set selectedItems to selection
		(* Check wether something selected *)
		if ((count of selectedItems) is greater than 0) then
			(* walk through items *)
			repeat with oneItem in selectedItems
				(* Do dry-run to determine file count *)
				set fileCount to 0
				tell me to createChecksums(oneItem, true)
				log "fileCount: " & fileCount
				if (fileCount is greater than 23) then
					(* Ask user if he's sure that's what he wants to do *)
					display dialog dialogTooMuchFilesMessage01 & fileCount & dialogTooMuchFilesMessage02 buttons {dialogTooMuchFilesYes, dialogTooMuchFilesNo} default button 2
					set returnedButton to button returned of result
					if (returnedButton is not dialogTooMuchFilesYes) then
						tell me to exit repeat #quit
					end if
				end if
				(* Now actually create the checksums *)
				set fileCount to 0
				tell me to createChecksums(oneItem, false)
			end repeat (* loop through sel. files *)
			(* If something went wrong -> tell the user *)
			if ((count of characters of logMessage) is greater than 0) then
				display dialog dialogErrorOccurredMessage & logMessage buttons {dialogErrorOccurredButton} default button 1
			end if
		else
			display dialog dialogNothingSelectedMessage buttons dialogNothingSelectedButton
		end if
	end tell (* tell finder *)
end run

(* subroutine for creating md5 checksums *)
on createChecksums(finderItem, dryRun)
	try
		(* check wether it's a file or a folder, if file create an additional md5 checksum file with similar name *)
		tell application "Finder"
			set theItemClass to class of finderItem
			set theItemName to name of finderItem
			set theItemContainer to (container of finderItem) as alias
			set itemPath to POSIX path of (finderItem as alias)
			set itemContainerPath to POSIX path of theItemContainer
			if (theItemClass is folder) then (* not using type because it's localized, class is not *)
				(* do this recursive, now with the contents of this folder *)
				set aliasList to every item of finderItem (* use of "every file" would result in a list only containing the files *)
				if ((count of aliasList) is greater than 0) then
					repeat with oneItem in items of aliasList
						tell me to createChecksums(oneItem, dryRun)
					end repeat
				end if
			else
				(* Don't create checksums of checksums! *)
				if (theItemName does not end with ".md5") then
					set fileCount to fileCount + 1
					(* If dry-running, only increment file counter *)
					if (dryRun is not true) then
						(* create checksum for file *)
						set checksumFileName to theItemName & ".md5"
						set cmd to "cd \"" & itemContainerPath & "\" && md5 -q \"" & theItemName & "\" | head -1 > \"" & checksumFileName & "\""
						log "Creating md5 checksum with this command: " & cmd
						try
							tell AppleScript to set cmdRes to do shell script cmd
						on error (e)
							log "This error occured while creating the checksum file: " & e
						end try
						log "Creating checksum returned: " & cmdRes
					end if (* if dryRun *)
				end if
			end if (* if file *)
		end tell (* tell finder *)
	on error (e)
		set logMessage to logMessage & "Error: " & e & return
	end try
end createChecksums

on systemLanguage()
	set cmd to "defaults read NSGlobalDomain AppleLanguages | grep -v \"^(\" | grep -v \"^)\" | head -1 | cut -d, -f1 | awk '{printf $1}'"
	
	try
		tell AppleScript to set languageCode to do shell script cmd
	on error (e)
		log "WTF? Cannot determine language code."
	end try
	
	if (languageCode is "de") then
		log "Deutsch"
	else if (languageCode is "en") then
		log "English"
	else
		log "WTF?"
	end if
	
	return languageCode
end systemLanguage





































































































(* nothing here *)